--------------------------------------------------------------------------------
local type = type
local string = string
local table = table
local s_gmatch = string.gmatch
local s_len = string.len
local t_insert = table.insert
local t_remove = table.remove
--------------------------------------------------------------------------------
local TEXTURE_GROUP_NAME = "numeric"
--------------------------------------------------------------------------------
-- All textures must have same sizeY.
local function gettextures( symbols )
	local txgroup = assert( common.GetAddonRelatedTextureGroup( TEXTURE_GROUP_NAME ), "texture group not found." )
	local textures, sizes = {}, {}
	local texture, size
	
	for symbol in s_gmatch( symbols, "." ) do
		texture = txgroup:GetTexture( symbol )
		size = widgetsSystem.GetTextureSize( texture )

		textures[ symbol ] = texture
		sizes[ symbol ] = size.sizeX
	end
	
	return textures, sizes, size.sizeY
end
--------------------------------------------------------------------------------
local textures, sizes, TEXTURE_SIZE = gettextures[[:.+-1234567890MGk]]
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
local SYMBOL_WIDGET_NAME = "Symbol"
local desc
--------------------------------------------------------------------------------
local function new( name )
	local widget
	
	if not desc then
		widget = mainForm:GetChildChecked( name, false )
		desc = widget:GetWidgetDesc()
	else
		widget = mainForm:CreateWidgetByDesc( desc )
	end

	return widget
end
--------------------------------------------------------------------------------
local function get( self, k )
	local widget = self[ k ]
	
	if not widget then
		widget = t_remove( self.pool )
		
		if not widget then
			widget = new( SYMBOL_WIDGET_NAME )
			self:AddChild( widget )
		end
		
		widget:SetForegroundColor( self.color )
		t_insert( self, widget )
	end
	
	return widget
end
--------------------------------------------------------------------------------
local function store( self, k )
	local widget = t_remove( self, k )
	t_insert( self.pool, widget )
	widget:Show( false )
end
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
Class( "widgetNumericTextView", {
	sizeY = 16,
	color = { a = 1, r = 1, g = 0.25, b = 0 },
} )
--------------------------------------------------------------------------------
function widgetNumericTextView:Init()
	self.pool = {}
	return self
end
--------------------------------------------------------------------------------
function widgetNumericTextView:SetSize( sizeY )
	local sizes = sizes
	local sizeX
	local ratio = sizeY / TEXTURE_SIZE
	local posX = 0
	
	for _, widget in ipairs( self ) do
		sizeX = sizes[ c ] * ratio
		widget:SetPlacementPlain{ sizeX = sizeX, sizeY = sizeY, posX = posX }
		posX = posX + sizeX
	end
	
	self:SetPlacementPlain{ sizeX = posX, sizeY = sizeY }
	
	self.sizeY = sizeY
end
--------------------------------------------------------------------------------
function widgetNumericTextView:SetColor( color )
	for _, widget in ipairs( self ) do
		widget:SetForegroundColor( color )
	end
	
	self.color = color
end
--------------------------------------------------------------------------------
-- not string: clear text
function widgetNumericTextView:SetText( text )
	local valid = type( text ) == "string"
	local k = 1
	
	if valid then
		local textures = textures
		local sizes = sizes
		local widget
		local texture
		local sizeX
		local sizeY = self.sizeY
		local ratio = sizeY / TEXTURE_SIZE
		local posX = 0
		
		for c in s_gmatch( text, "." ) do
			texture = textures[ c ]
			sizeX = sizes[ c ] * ratio
			
			widget = get( self, k )
			widget:Show( true )
			widget:SetForegroundTexture( texture )
			widget:SetBackgroundTexture( texture )
			widget:SetPlacementPlain{ sizeX = sizeX, sizeY = sizeY, posX = posX }
			
			posX = posX + sizeX
			k = k + 1
		end
		
		self:SetPlacementPlain{ sizeX = posX, sizeY = sizeY }
	end

	while self[ k ] do store( self, k ) end
	
	self:Show( valid )
	self.text = text
end
--------------------------------------------------------------------------------